//MCU = ATtiny13a
//F_CPU = 9600000


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


/* Подключение файла с различными обьявлениями макросов
 * для дальнейшего использования их в программе.
 */
#include "definitions.h"


#define NEW_MODE (mode & 0b00111111) == 0 		// Проверка на начало нового режима

int mode = 0;
/*	описание битов mode:
    0-3 bit - этап
    4 bit - направление (влево/вправо)
    5 bit - счет/сдвиг
    6 bit -	движение по строке/диагонали
    7-9 bit - стартовая позиция
*/

// Массив стартовых позиций для режима счета
uchar start_pos[] = {0b00000001,
                     0b00001111,
                     0b00010001,
                     0b00011111,
                     0b01111111};


void updateRegisters(uchar row, uchar column);  // записать рисунок на 8 последновательных регистров
void loadByte(uchar byte);          // Загрузить байт в регистр

void next_step(uchar *row, uchar *column);				// Перейти к следующему "шагу" в светодинамическом рисунке.
uchar make_move(uchar port);		// Определение режима: сдвиг или счет.

uchar shift(uchar port);			// Определение направления сдвига
uchar shift_left(uchar port);		// Сдвиг влево
uchar shift_right(uchar port);		// Сдвиг вправо

uchar count(uchar port);			// Определение направления счета
uchar count_left(uchar port);		// Счет влево
uchar count_right(uchar port);		// Счет вправо


/* Тактирование от таймера
 * Частота тактирования (clock) вычисляется по следующей формуле:
 * clock = F_CPU / tim_pres / (OCR0A + 1), где tim_pre - предделитель таймера.
 * В нашем случае F_CPU = 9600000, tim_pre  1024, и OCR0A = 255:
 * clock = 9600000 / 1024 / 256 = 36.6 Гц
 */
ISR(TIM0_COMPA_vect) {
    static uchar cnt = 0;
    static uchar row, column;

    cnt++;
    if(cnt == 8) {                       // Делитель частоты тактирования рисунка
        cnt = 0;
        next_step(&row, &column);        // Перейти к следующему "шагу" в светодинамическом рисунке.
        updateRegisters(row, column);    // Записать рисунок на 8 последновательных регистров
        STATE_LOAD;                      // Выгрузить на выходы новое состояние регистров
    }
}


int main(void) {
    PORTB = 0b11111000;
    DDRB =  0b00000111;

    TCCR0A = 0x02;              // Режим сброс по совпадению
    TCCR0B = 0x05;              // Предделитель на 1024

    OCR0A = 255;
    TIMSK0 = 1 << OCIE0A;       // Разрешение прерываний таймера по совпадению

    sei();
    for(;;) {

    }
}


// записать рисунок на 8 последновательных регистров
void updateRegisters(uchar row, uchar column)
{
    uchar rowCnt;
    for(rowCnt = 0; rowCnt < 8; ++rowCnt) {     // Последоавтельная запись 8 байт на регистры
        if((row & 0x80) == 0)
            loadByte(0);
        else
            loadByte(column);
        row <<= 1;
    }
}


// Загрузить байт в регистр
void loadByte(uchar byte) {
    uchar i;
    for(i = 0; i < 8; ++i) {        // Последовательная передача 8 бит на регистр
        if((byte & 0x80) == 0)
            DATA_LOW;
        else
            DATA_HIGH;
        SHIFT_CLOCK;                // Тактирующий импульс после каждого бита
        byte <<= 1;
    }
}


// Перейти к следующему "шагу" в светодинамическом рисунке.
void next_step(uchar *row, uchar *column)
{
    if(bit_is_clear(mode,6)) {				// Движение по столбцам
        if( NEW_MODE ) {					// Если новый режим, то установить начальное значение
            *row = 0xFF;
            *column = start_pos[mode>>7];	// mode >> 8 - индекс массива стартовых позиций
        }
        else
            *column = make_move(*column);
    }
    else {                           		// Движение по строкам
        if( NEW_MODE ) {					// Если новый режим, то установить начальное значение
            *column = 0xFF;
            *row = start_pos[mode>>7];		// mode >> 8 - индекс массива стартовых позиций
        }
        else
            *row = make_move(*row);
    }

    mode++;
    if(mode == 128 * sizeof (start_pos))	// Зацикливание рисунков (когда все режимы показаны - повторить снова)
        mode = 0;
}


// Определение режима: сдвиг или счет.
uchar make_move(uchar port) {
    if(bit_is_clear(mode, 5)) {				// если 5 бит в mode == 0, то режим сдвига
        if((mode & 0b00011111) == 31)		// вместо 32 шага сдвига - установка порта в 0 (что бы режим счета начинался с нуля)
            return 0;
        else
            return shift(port);
    }
    else
        return count(port);					// если 5 бит в mode == 1, то режим счета
}


// Режим счета
uchar count(uchar port) {
    if(bit_is_set(mode, 4))
        return count_right(port);		// если 4 бит в mode == 1, то режим счета вправо
    else
        return count_left(port);		// если 4 бит в mode == 0, то режим счета влево
}


// Счет вправо
uchar count_right(uchar port) {
    port >>= 1;							// сдвиг вправо
    if(bit_is_clear(mode, 3))
        port |= 0b10000000;
    return port;
}


// Счет влево
uchar count_left(uchar port) {
    port <<= 1;							// сдвиг влево
    if(bit_is_clear(mode, 3))
        port |= 0b00000001;
    return port;
}


// Режим сдвига
uchar shift(uchar port) {
    if(bit_is_set(mode, 4))
        return shift_right(port);		// сдвиг вправо, если 4 бит в mode единица
    else
        return shift_left(port);		// иначе сдвиг влево
}


// Сдвиг влево
uchar shift_left(uchar port) {
    if(bit_is_set(port, 7)) {	// если самый левый бит единица, то после сдвига в единицу нужно установить самый правый бит
        port <<= 1;				// сдвиг влево на 1
        port |= 0b00000001;
    }
    else
        port <<= 1;
    return port;
}


// Сдвиг вправо
uchar shift_right(uchar port) {
    if(bit_is_set(port, 0)) {	// если самый правый бит единица, то после сдвига в единицу нужно установить самый левый бит
        port >>= 1;				// сдвиг вправо на 1
        port |= 0b10000000;
    }
    else
        port >>= 1;
    return port;
}

